
; ===========================================================================
;   Eternity II Monte Carlo puzzle solver, v1.0, (c) 2007 Miroslav Nemecek
; ===========================================================================

			.586
			.model	flat
			.code

; ---------------------------------------------------------------------------
;                               Constants
; ---------------------------------------------------------------------------

DIR_UP		EQU		0					; UP direction
DIR_RIGHT	EQU		1					; RIGHT direction
DIR_DOWN	EQU		2					; DOWN direction
DIR_LEFT	EQU		3					; LEFT direction

BRD_WIDTH	EQU		16					; width of board
BRD_HEIGHT	EQU		16					; height of board
BRD_SIZE	EQU		(BRD_WIDTH*BRD_HEIGHT) ; size of board

BLOCKS		EQU		(BRD_WIDTH*BRD_HEIGHT) ; number of blocks (=256)

PERIOD		EQU		1000				; view period

; ---------------------------------------------------------------------------
;                              Structures
; ---------------------------------------------------------------------------

; --------- Atom (12 bytes) (on change update CAtom in EternityIIMC.h)

CAtom		STRUCT

Next		dd		?					; next atom in the list (NULL=none)
										; (it MUST be first entry, hardcoded!)
Block		db		?					; block number
Spin		db		?					; block spin (quarter-turns clockwise)
Res			dw		?					; ...padding
Color		db		4 dup(?)			; colors in all directions

CAtom		ENDS
		
; --------- Field (16 bytes) (on change update CField in EternityIIMC.h)

CField		STRUCT

Atom		dd		?					; atom in this field (NULL=none)
Color		db		4 dup(?)			; required colors of this field
ColMask		db		4 dup(?)			; color mask of this field
Edge		db		?					; edge flags (see below)
Res			db		3 dup(?)			; ...padding

CField		ENDS

CField_BITS	EQU		4					; CField size in bits

; --------- Edge flags

EDGE_UP		EQU		1					; edge in UP direction
EDGE_RIGHT	EQU		2					; edge in RIGHT direction
EDGE_DOWN	EQU		4					; edge in DOWN direction
EDGE_LEFT	EQU		8					; edge in LEFT direction
EDGE_HINT	EQU		16					; field contains hint

OFFW		EQU		(BRD_WIDTH*SIZE CField) ; offset of next row
OFF1		EQU		(SIZE CField)		; offset of next field

; ---------------------------------------------------------------------------
;                              Externals
; ---------------------------------------------------------------------------

extern		_DispState:NEAR
extern		_SaveBest:NEAR

extern		_RandSeed:DWORD
extern		_Board:CField
extern		_Used:DWORD
extern		_UsedNum:DWORD
extern		_MaxUsed:DWORD
extern		_AtomList:DWORD
extern		_AtomLast:DWORD
extern		_BreakCount:DWORD
extern		_BreakOK:DWORD
extern		_BreakRe:DWORD
extern		_BreakBlk:DWORD

; ===========================================================================
;                               Solve puzzle
; ===========================================================================

public		_Attack

; --------- Push registers

_Attack:	pusha						; push all registers

; --------- Prepare counter of view period

			mov		ebp,PERIOD			; EBP <- view period

; --------- Generate random position (->EAX)

AttackPos:	mov		eax,214013			; EAX <- multiple constant 1
			mul		dword ptr [_RandSeed] ; multiple with contanst 1
			add		eax,2531011			; add constant 2
			mov		[_RandSeed],eax		; store new random seed
			shr		eax,32-8			; EAX <- random number 0 to 255

; --------- Check if this field (EAX) is free

			shl		eax,CField_BITS		; EAX <- offset in board
			add		eax,offset _Board	; EAX <- address of field
			cmp		[eax].CField.Atom,0	; is it free entry?
			jne		AttackPos			; generate next entry

; --------- Prepare colors (-> ESI) and color mask (-> EDI) to search

			mov		esi,dword ptr [eax].CField.Color ; ESI <- colors
			mov		edi,dword ptr [eax].CField.ColMask ; EDI <- color mask
			and		esi,edi				; mask colors
			
; --------- Find suitable atom (-> ECX, EBX = previous atom)

			mov		ecx,offset _AtomList ; ECX <- list of atoms
AttackAtom:	mov		ebx,ecx				; EBX <- save previous atom
			mov		ecx,[ebx]			; ECX <- next atom
			or		ecx,ecx				; is it end of list?
			jz		AttackBreak			; atom not found, break position

; --------- Check color of atom

			mov		edx,dword ptr [ecx].CAtom.Color ; EDX <- atom colors
			and		edx,edi				; EDX <- mask colors
			cmp		edx,esi				; check colors
			jne		AttackAtom			; invalid colors, get next atom
			
; --------- Initialize break counter

			mov		edx,[_BreakOK]		; EDX <- break counter
			mov		[_BreakCount],edx	; initialize break counter
		
; --------- Detach atom from the list

			mov		edx,[ecx]			; EDX <- next atom
			or		edx,edx				; is it last entry in the list?
			jz		AttackAtom4			; it is last entry in the list
			mov		[ebx],edx			; link next entry to previous

; --------- Attach atom to end of list

			xor		[ecx],edx			; delete link to next entry
			mov		edx,[_AtomLast]		; EDX <- last entry in the list
			mov		[edx],ecx			; link this atom to last entry of the list
			mov		[_AtomLast],ecx		; new last entry in the list

; --------- Check if this block is free

AttackAtom4:movzx	edx,[ecx].CAtom.Block ; EDX <- block number
			mov		edx,[_Used+edx*4]	; EDX <- old field
			or		edx,edx				; is this block free?
			jz		AttackSet			; block is free
			
; --------- Detach atom ECX from the old field EDX

			mov		bl,[edx].CField.Edge ; BL <- edge flags
			test	bl,EDGE_UP			; edge in UP direction?
			jnz		AttackFree2			; there is edge in UP direction
			mov		[edx-OFFW].CField.ColMask[DIR_DOWN],0 ; clear mask
AttackFree2:test	bl,EDGE_RIGHT		; edge in RIGHT direction?
			jnz		AttackFree3			; there is edge in RIGHT direction
			mov		[edx+OFF1].CField.ColMask[DIR_LEFT],0 ; clear mask
AttackFree3:test	bl,EDGE_DOWN		; edge in RIGHT direction?
			jnz		AttackFree4			; there is edge in RIGHT direction
			mov		[edx+OFFW].CField.ColMask[DIR_UP],0 ; clear mask
AttackFree4:test	bl,EDGE_LEFT		; edge in RIGHT direction?
			jnz		AttackFree5			; there is edge in RIGHT direction
			mov		[edx-OFF1].CField.ColMask[DIR_RIGHT],0 ; clear mask
AttackFree5:and		[edx].CField.Atom,0	; delete atom from the field
			dec		dword ptr [_UsedNum] ; decrease number of blocks
			
; --------- Store atom ECX into new field EAX
			
AttackSet:	mov		edx,dword ptr [ecx].CAtom.Color ; EDX <- colors
			mov		bl,[eax].CField.Edge ; BL <- edge flags
			test	bl,EDGE_UP			; edge in UP direction?
			jnz		AttackSet2			; there is edge in UP direction
			mov		[eax-OFFW].CField.Color[DIR_DOWN],dl ; set color
			mov		[eax-OFFW].CField.ColMask[DIR_DOWN],0ffh ; set mask
AttackSet2:	test	bl,EDGE_RIGHT		; edge in RIGHT direction?
			jnz		AttackSet3			; there is edge in RIGHT direction
			mov		[eax+OFF1].CField.Color[DIR_LEFT],dh ; set color
			mov		[eax+OFF1].CField.ColMask[DIR_LEFT],0ffh ; set mask
AttackSet3:	shr		edx,16				; rotate colors
			test	bl,EDGE_DOWN		; edge in RIGHT direction?
			jnz		AttackSet4			; there is edge in RIGHT direction
			mov		[eax+OFFW].CField.Color[DIR_UP],dl ; set color
			mov		[eax+OFFW].CField.ColMask[DIR_UP],0ffh ; set mask
AttackSet4:	test	bl,EDGE_LEFT		; edge in RIGHT direction?
			jnz		AttackSet5			; there is edge in RIGHT direction
			mov		[eax-OFF1].CField.Color[DIR_RIGHT],dh ; set color
			mov		[eax-OFF1].CField.ColMask[DIR_RIGHT],0ffh ; set mask
AttackSet5:	mov		[eax].CField.Atom,ecx ; store atom into field

; --------- Set used flag

			movzx	edx,[ecx].CAtom.Block ; EDX <- block number
			mov		[_Used+edx*4],eax	; set new field
			inc		dword ptr [_UsedNum] ; increase number of blocks

; --------- View period

			dec		ebp					; counting view period
			jz		AttackView			; do view

; --------- New level

			mov		edx,[_UsedNum]		; EDX <- number of used blocks
			cmp		edx,[_MaxUsed]		; check maximal used blocks
			jbe		AttackPos			; next block
			mov		[_MaxUsed],edx		; set new maximal used blocks
			cmp		edx,BLOCKS			; all blocks?
			je		AttackStop			; stop
		
; --------- Save new best position

			pusha						; push all registers
			call	_SaveBest			; save maximum
			popa						; pop all registers
AttackSet8:	jmp		AttackPos			; next block

; --------- Invalid position, prepare field to free (-> EDX)

AttackBreak:dec		ebp					; counting view period
			jz		AttackView			; do view
			dec		[_BreakCount]		; break counter
			jnz		AttackPos			; next try
			mov		ecx,[_BreakRe]		; EDX <- break counter
			mov		[_BreakCount],ecx	; reinitialize break counter


;AttackBrk11:mov		bl,[eax].CField.Edge ; BL <- edge flags
;			or		bl, byte ptr [_RandSeed] ; add some randomize
;			test	bl,EDGE_UP			; edge in UP direction?
;			jnz		AttackBrk12			; there is edge in UP direction
;			lea		edx,[eax-OFFW]		; EDX <- field up
;			test	[edx].CField.Edge,EDGE_HINT ; hint?
;			jnz		AttackBrk12			; cannot move hint block
;			cmp		dword ptr [edx].CField.Atom,0 ; is there an atom?
;			jne		AttackBrk15			; there is valid atom
;AttackBrk12:test	bl,EDGE_RIGHT		; edge in RIGHT direction?
;			jnz		AttackBrk13			; there is edge in RIGHT direction
;			lea		edx,[eax+OFF1]		; EDX <- field right
;			test	[edx].CField.Edge,EDGE_HINT ; hint?
;			jnz		AttackBrk13			; cannot move hint block
;			cmp		dword ptr [edx].CField.Atom,0 ; is there an atom?
;			jne		AttackBrk15			; there is valid atom
;AttackBrk13:test	bl,EDGE_DOWN		; edge in DOWN direction?
;			jnz		AttackBrk14			; there is edge in DOWN direction
;			lea		edx,[eax+OFFW]		; EDX <- field down
;			test	[edx].CField.Edge,EDGE_HINT ; hint?
;			jnz		AttackBrk14			; cannot move hint block
;			cmp		dword ptr [edx].CField.Atom,0 ; is there an atom?
;			jne		AttackBrk15			; there is valid atom
;AttackBrk14:test	bl,EDGE_LEFT		; edge in LEFT direction?
;			jnz		AttackBrk1			; there is edge in LEFT direction
;			lea		edx,[eax-OFF1]		; EDX <- field left
;			test	[edx].CField.Edge,EDGE_HINT ; hint?
;			jnz		AttackBrk1			; cannot move hint block
;			cmp		dword ptr [edx].CField.Atom,0 ; is there an atom?
;			je		AttackBrk1			; there is not valid atom

; --------- Detach atom from the old field EAX

;AttackBrk15:mov		ecx,[edx].CField.Atom ; EDX <- atom
;			mov		bl,[edx].CField.Edge ; BL <- edge flags
;			test	bl,EDGE_UP			; edge in UP direction?
;			jnz		AttackBrk16			; there is edge in UP direction
;			mov		[edx-OFFW].CField.ColMask[DIR_DOWN],0 ; clear mask
;AttackBrk16:test	bl,EDGE_RIGHT		; edge in RIGHT direction?
;			jnz		AttackBrk17			; there is edge in RIGHT direction
;			mov		[edx+OFF1].CField.ColMask[DIR_LEFT],0 ; clear mask
;AttackBrk17:test	bl,EDGE_DOWN		; edge in RIGHT direction?
;			jnz		AttackBrk18			; there is edge in RIGHT direction
;			mov		[edx+OFFW].CField.ColMask[DIR_UP],0 ; clear mask
;AttackBrk18:test	bl,EDGE_LEFT		; edge in RIGHT direction?
;			jnz		AttackBrk19			; there is edge in RIGHT direction
;			mov		[edx-OFF1].CField.ColMask[DIR_RIGHT],0 ; clear mask
;AttackBrk19:movzx	ebx,[ecx].CAtom.Block ; EBX <- block number
;			xor		ecx,ecx				; EDX <- 0
;			xchg	[edx].CField.Atom,ecx ; delete atom from the field
;			dec		dword ptr [_UsedNum] ; decrease number of blocks
;			xor		[_Used+ebx*4],edx	; delete old field


;AttackBrk1:


; --------- Generate random position (->EAX)

			mov		ecx,1
			xchg	ecx,[_BreakBlk]		; ECX <- number of breaked blocks
			cmp		ecx,[_UsedNum]		; check number of blocks
			jbe		AttackBrk2			; number of blocks is OK
			mov		ecx,[_UsedNum]		; ECX <- limit number of blocks			
AttackBrk2:	mov		eax,214013			; EAX <- multiple constant 1
			mul		dword ptr [_RandSeed] ; multiple with contanst 1
			add		eax,2531011			; add constant 2
			mov		[_RandSeed],eax		; store new random seed
			shr		eax,32-8			; EAX <- random number 0 to 255

; --------- Check if this field (EAX) is free

			shl		eax,CField_BITS		; EAX <- offset in board
			add		eax,offset _Board	; EAX <- address of field
			cmp		[eax].CField.Atom,0	; is it engaged entry?
			je		AttackBrk2			; generate engaged entry
			test	[eax].CField.Edge,EDGE_HINT ; hint?
			jnz		AttackBrk2			; cannot free hint block
			
			

; --------- Detach atom from the old field EAX

AttackBrk5:	mov		edx,[eax].CField.Atom ; EDX <- atom
			mov		bl,[eax].CField.Edge ; BL <- edge flags
			test	bl,EDGE_UP			; edge in UP direction?
			jnz		AttackBrk6			; there is edge in UP direction
			mov		[eax-OFFW].CField.ColMask[DIR_DOWN],0 ; clear mask
AttackBrk6:	test	bl,EDGE_RIGHT		; edge in RIGHT direction?
			jnz		AttackBrk7			; there is edge in RIGHT direction
			mov		[eax+OFF1].CField.ColMask[DIR_LEFT],0 ; clear mask
AttackBrk7:	test	bl,EDGE_DOWN		; edge in RIGHT direction?
			jnz		AttackBrk8			; there is edge in RIGHT direction
			mov		[eax+OFFW].CField.ColMask[DIR_UP],0 ; clear mask
AttackBrk8:	test	bl,EDGE_LEFT		; edge in RIGHT direction?
			jnz		AttackBrk9			; there is edge in RIGHT direction
			mov		[eax-OFF1].CField.ColMask[DIR_RIGHT],0 ; clear mask
AttackBrk9:	movzx	ebx,[edx].CAtom.Block ; EBX <- block number
			xor		edx,edx				; EDX <- 0
			xchg	[eax].CField.Atom,edx ; delete atom from the field
			dec		dword ptr [_UsedNum] ; decrease number of blocks
			xor		[_Used+ebx*4],eax	; delete old field
			loop	AttackBrk2			; next block
			jmp		AttackPos			; new block

; --------- View

AttackView:	pusha						; push all registers
			call	_DispState			; display status
			popa						; pop all registers
			mov		ebp,PERIOD			; EBP <- view period
			jmp		AttackPos			; next block
			
; --------- Pop registers

AttackStop:	popa						; pop all registers
			ret

BreakCount	db		255					; break counter

			END
