
//////////////////////////////////////////////////////////////////////////////
//                    Eternity II Block Set Solver
//////////////////////////////////////////////////////////////////////////////

// Compilation flags (defined with switch /Dxxx)
// #define		_DEBUG				// Flag, do some internal checks
// #define		MINI				// Flag, use HINT 1 ("mini") puzzle
// #define		MIDI				// Flag, use HINT 2 ("midi") puzzle
// #define		MAXI				// Flag, use full ("maxi") puzzle
// #define		USEHINTS			// Flag, use hints 1 and 2 in "maxi" puzzle

#pragma warning(disable : 4996)		// "printf" was declared deprecated

//////////////////////////////////////////////////////////////////////////////
// Includes

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#ifdef _DEBUG
#define ASSERT(f)			{ if (!(f)) _asm{int 3}; }
#else	// _DEBUG
#define ASSERT(f)			((void)0)
#endif	// _DEBUG

//////////////////////////////////////////////////////////////////////////////
// Constants

#ifdef MINI
#define		WIDTH	6				// width of area
#define		HEIGHT	6				// height of area
#define		COLORS	8				// number of colors (with border)
#define		OCOLMIN	1				// index of beginning outer color
#define		OCOLMAX	4				// index of ending outer color
#define		ICOLMIN	5				// index of beginning inner color
#define		ICOLMAX	7				// index of ending inner color
#endif // MINI

#ifdef MIDI
#define		WIDTH	12				// width of area
#define		HEIGHT	6				// height of area
#define		COLORS	9				// number of colors (with border)
#define		OCOLMIN	1				// index of beginning outer color
#define		OCOLMAX	4				// index of ending outer color
#define		ICOLMIN	5				// index of beginning inner color
#define		ICOLMAX	8				// index of ending inner color
#endif // MIDI

#ifdef MAXI
#define		WIDTH	16				// width of area
#define		HEIGHT	16				// height of area
#define		COLORS	23				// number of colors (with border)
#define		OCOLMIN	1				// index of beginning outer color
#define		OCOLMAX	5				// index of ending outer color
#define		ICOLMIN	6				// index of beginning inner color
#define		ICOLMAX	22				// index of ending inner color
#endif // MAXI

#define		BLOCKS	(WIDTH*HEIGHT)	// number of blocks (=36, 72, 256)
#define		EDGECOL	0				// edge color
#define		CBLOCKS	4				// corner blocks
#define		EBLOCKSH (WIDTH-2)		// edge blocks on horizontal sides (=4, 10, 14)
#define		EBLOCKSV (HEIGHT-2)		// edge blocks on vertical sides (=4, 4, 14)
#define		EBLOCKS	(2*EBLOCKSH+2*EBLOCKSV)	// edge blocks (=16, 28, 56)
#define		CEBLOCK	(CBLOCKS+EBLOCKS) // corner+edge blocks
#define		COLBITS	8				// number of bits per color
#define		COLNUM	(1 << COLBITS)	// number of colors in hash table (=256)
#define		COLNUM2	(COLORS*COLNUM)	// number of colors in double hash table (=2048, 2048, 5888)

#define		CORNINI	24				// number of corner initial spins

#define		OCOLNUM	(OCOLMAX-OCOLMIN+1) // number of outer colors (=4, 5)
#define		ICOLNUM	(ICOLMAX-ICOLMIN+1) // number of inner colors (=3, 17)

// Hints
#ifdef MAXI
#define		H0BLOCK	138				// hint 0 block (=block 139)
#define		H0SPIN	2				// hint 0 spin (clockwise)
#define		H0POSX	7				// hint 0 position X
#define		H0POSY	8				// hint 0 position Y
#define		H0POS	(H0POSX+H0POSY*WIDTH) // hint 0 position (I,8)

#define		H0COLD	14				// hint 0 color down (rotated)
#define		H0COLL	8				// hint 0 color left (rotated)
#define		H0COLU	9				// hint 0 color up (rotated)
#define		H0COLR	14				// hint 0 color right (rotated)

#ifdef USEHINTS
#define		H1BLOCK	180				// hint 1 block (=block 181)
#define		H1SPIN	3				// hint 1 spin (clockwise)
#define		H1POSX	2				// hint 1 position X
#define		H1POSY	13				// hint 1 position Y
#define		H1POS	(H1POSX+H1POSY*WIDTH) // hint 1 position (N,3)

#define		H1COLD	19				// hint 1 color down (rotated)
#define		H1COLL	8				// hint 1 color left (rotated)
#define		H1COLU	16				// hint 1 color up (rotated)
#define		H1COLR	14				// hint 1 color right (rotated)

#define		H2BLOCK	254				// hint 2 block (=block 255)
#define		H2SPIN	3				// hint 2 spin (clockwise)
#define		H2POSX	13				// hint 2 position X
#define		H2POSY	2				// hint 2 position Y
#define		H2POS	(H2POSX+H2POSY*WIDTH) // hint 2 position (C,14)

#define		H2COLD	13				// hint 2 color down (rotated)
#define		H2COLL	16				// hint 2 color left (rotated)
#define		H2COLU	13				// hint 2 color up (rotated)
#define		H2COLR	20				// hint 2 color right (rotated)

#define		HINTNUM	3				// number of hints
#else // USEHINTS
#define		HINTNUM	1				// number of hints
#endif // USEHINTS
#else // MAXI
#define		HINTNUM	0				// number of hints
#endif // MAXI

#ifdef _DEBUG
#define		HISTSIZE 1000000		// history size
#define		HISTHASH 0x10000		// size of history hash table (64 K)
#endif // _DEBUG

// Directions and turns - from down side to the right (clockwise)
enum {
	DIR_DOWN	= 0,				// DOWN direction
	DIR_LEFT	= 1,				// LEFT direction
	DIR_UP		= 2,				// UP direction
	DIR_RIGHT	= 3,				// RIGHT direction
};

//////////////////////////////////////////////////////////////////////////////
// Structures

// Color (size 4 bytes)
struct CColor {
	union {
		char			Color[4];	// colors in all directions
		struct {
			char		ColD;		// color in DOWN direction
			char		ColL;		// color in LEFT direction
			char		ColU;		// color in UP direction
			char		ColR;		// color in RIGHT direction
		};
		DWORD			ColorDW;	// color as DWORD
	};
};

// Block (size 16 bytes, colors are in inverse order)
struct CBlock {
	union {
		char		Color[8];		// colors in all directions
		struct {
			char		ColD;		// color in DOWN direction
			char		ColR;		// color in RIGHT direction
			char		ColU;		// color in UP direction
			char		ColL;		// color in LEFT direction

			char		ColD2;		// color in DOWN direction
			char		ColR2;		// color in RIGHT direction
			char		ColU2;		// color in UP direction
			char		ColL2;		// color in LEFT direction
		};
		DWORD		ColorDW[2];		// colors as DWORD
	};
	char			Flags;			// block flags (see below)
	BYTE			Block;			// block index
	char			Spin0;			// original spin (clockwise)
	char			Spin;			// current spin (clockwise)
	struct CBlock*	Next;			// next block in hash table (NULL=none)
};

// Block flags
#define		HINT	1				// block is hint
#define		LOCKED	2				// block is locked
#define		USED	4				// block is used

// Statistic (size 4 bytes)
struct CStat {
	union {
		char		Num[4];			// statistic in all directions
		struct {
			char		NumD;		// statistic in DOWN direction
			char		NumL;		// statistic in LEFT direction
			char		NumU;		// statistic in UP direction
			char		NumR;		// statistic in RIGHT direction
		};
		DWORD		NumDW;			// statistic as DWORD
	};
};

// Field (size 8 bytes, colors are in inverse order)
struct CField {
	union {
		char		Color[4];		// colors in all directions
		struct {
			char		ColD;		// color in DOWN direction
			char		ColR;		// color in RIGHT direction
			char		ColU;		// color in UP direction
			char		ColL;		// color in LEFT direction
		};
		DWORD		ColorDW;		// colors as DWORD
	};
	struct CBlock*	Block;			// pointer to block
};

//////////////////////////////////////////////////////////////////////////////
// Externals for assembler

extern "C" void DispState();		// display state
extern "C" void SaveBest();			// Save best board
extern "C" void Attack();			// solve puzzle
#ifdef _DEBUG
extern "C" void CheckStat();		// debug check color statistic
#endif // _DEBUG

extern "C" unsigned __int64	RandSeed; // seed of random generator
extern "C" int Variants;			// number of variants
extern "C" int Level;				// current level
extern "C" int MaxLevel;			// maximal level

extern "C" struct CBlock Block[BLOCKS]; // blocks
extern "C" struct CStat Stat[COLORS]; // color statistic
extern "C" struct CStat StatSave[COLORS]; // saved color statistic
extern "C" int Repeat;				// repeat counter
extern "C" struct CBlock* CheckBlock[WIDTH]; // blocks for edge check

#ifdef MAXI
extern "C" struct CBlock* Hint0;	// hint 0
#ifdef USEHINTS
extern "C" struct CBlock* Hint1;	// hint 1
extern "C" struct CBlock* Hint2;	// hint 2
#endif // USEHINTS
#endif // MAXI

extern "C" char CornIni[CORNINI*4]; // corner initial spins

extern "C" struct CBlock* EdgeL[OCOLNUM]; // start pointer of left-color edges
extern "C" struct CBlock* EdgeR[OCOLNUM]; // start pointer of right-color edges
extern "C" struct CBlock* EdgeD[OCOLNUM]; // start pointer of double-color edges

extern "C" char EdgeLNum[OCOLNUM];	// number of left-color edges
extern "C" char	EdgeRNum[OCOLNUM];	// number of right-color edges
extern "C" char	EdgeDNum[OCOLNUM];	// number of double-color edges

extern "C" struct CBlock* BlockD[ICOLNUM]; // start pointer of down-color block
extern "C" struct CBlock* BlockDL[ICOLNUM]; // start pointer of down-left-color block

extern "C" char BlockDNum[ICOLNUM];		// number of down-color blocks
extern "C" char BlockDLNum[ICOLNUM];	// number of down-left-color blocks

#ifndef MAXI
extern "C" struct CBlock* BlockC;		// start pointer of cross blocks
extern "C" char BlockCNum;				// number of cross blocks
#endif // MAXI

extern "C" char EdgeFree[4];			// free positions in all directions
extern "C" struct CStat EdgeLMin[OCOLNUM];	// minimal number of left-color edges
extern "C" struct CStat	EdgeRMin[OCOLNUM];	// minimal number of right-color edges
extern "C" struct CStat	EdgeLRMax[OCOLNUM];	// maximal number of edge pairs
extern "C" char	EdgeLRIni[OCOLNUM];		// initial number of edge pairs
extern "C" struct CStat	EdgeLRNum[OCOLNUM];	// current number of edge pairs
extern "C" struct CStat	EdgeLFree[OCOLNUM];	// number of free left-color edge positions
extern "C" struct CStat	EdgeRFree[OCOLNUM];	// number of free right-color edge positions

extern "C" struct CStat	BlockDDFree[ICOLNUM]; // number of free down-color block positions

extern "C" struct CField Board[BLOCKS];	// board
extern "C" struct CBlock* Hash[COLNUM2]; // block hash table without edges
extern "C" struct CBlock* HashR[COLNUM2]; // block hash table with right edge
extern "C" struct CBlock* HashD[COLNUM2]; // block hash table with down edge
extern "C" struct CBlock* HashDR[COLNUM2]; // block hash table with down and right edge

// History check
#ifdef _DEBUG
extern "C" BYTE Hist[HISTSIZE*BLOCKS/4]; // solution history (size 64 MB)
extern "C" BYTE Hist0[BLOCKS/4];		// current solution
extern "C" int HistFirst[HISTHASH];		// history hash - first entry (-1 = none, size 256 KB)
extern "C" int HistNext[HISTSIZE];		// history hash - next entry (-1 = none, size 4 MB)
extern "C" int Similar;					// number of similar solutions
extern "C" int HistNum;					// number of entries in history
#endif // _DEBUG

//////////////////////////////////////////////////////////////////////////////
// Block color definition (down, left, up and right direction)

struct CColor Color[BLOCKS] = {

#ifdef MINI
	{	6,	5,	5,	6,	},			// block 1 
	{	5,	6,	5,	6,	},			// block 2 
	{	5,	7,	5,	6,	},			// block 3 
	{	6,	7,	5,	6,	},			// block 4 
	{	5,	7,	5,	6,	},			// block 5 
	{	6,	7,	5,	5,	},			// block 6 
	{	0,	2,	6,	1,	},			// block 7 
	{	6,	5,	5,	6,	},			// block 8 
	{	5,	5,	6,	5,	},			// block 9 
	{	4,	0,	4,	6,	},			// block 10
	{	2,	0,	4,	5,	},			// block 11
	{	6,	4,	0,	1,	},			// block 12
	{	3,	5,	4,	0,	},			// block 13
	{	0,	0,	1,	3,	},			// block 14
	{	6,	3,	0,	2,	},			// block 15
	{	0,	2,	5,	4,	},			// block 16
	{	3,	5,	3,	0,	},			// block 17
	{	2,	0,	4,	6,	},			// block 18
	{	6,	5,	6,	5,	},			// block 19
	{	6,	3,	0,	4,	},			// block 20
	{	0,	2,	6,	4,	},			// block 21
	{	1,	0,	2,	5,	},			// block 22
	{	0,	3,	6,	2,	},			// block 23
	{	5,	5,	6,	6,	},			// block 24
	{	2,	0,	1,	6,	},			// block 25
	{	1,	0,	0,	1,	},			// block 26
	{	5,	5,	6,	5,	},			// block 27
	{	6,	1,	0,	3,	},			// block 28
	{	5,	5,	7,	7,	},			// block 29
	{	7,	6,	6,	6,	},			// block 30
	{	4,	5,	3,	0,	},			// block 31
	{	6,	5,	6,	6,	},			// block 32
	{	7,	5,	6,	6,	},			// block 33
	{	2,	0,	0,	3,	},			// block 34
	{	6,	6,	6,	5,	},			// block 35
	{	1,	1,	0,	0,	},			// block 36
#endif // MINI

#ifdef MIDI
	{	1,	5,	2,	0,	},			// block 1
	{	6,	6,	6,	5,	},			// block 2
	{	7,	7,	8,	5,	},			// block 3
	{	5,	2,	0,	1,	},			// block 4
	{	7,	6,	5,	6,	},			// block 5
	{	5,	6,	7,	7,	},			// block 6
	{	2,	3,	0,	0,	},			// block 7
	{	5,	5,	5,	5,	},			// block 8
	{	6,	6,	6,	8,	},			// block 9
	{	8,	3,	0,	4,	},			// block 10
	{	8,	8,	6,	5,	},			// block 11
	{	8,	6,	6,	7,	},			// block 12
	{	8,	5,	5,	6,	},			// block 13
	{	5,	8,	5,	5,	},			// block 14
	{	5,	5,	6,	7,	},			// block 15
	{	3,	0,	4,	5,	},			// block 16
	{	6,	3,	0,	1,	},			// block 17
	{	1,	5,	4,	0,	},			// block 18
	{	6,	6,	5,	5,	},			// block 19
	{	5,	2,	0,	3,	},			// block 20
	{	0,	2,	5,	1,	},			// block 21
	{	0,	2,	5,	2,	},			// block 22
	{	3,	0,	2,	5,	},			// block 23
	{	6,	5,	5,	6,	},			// block 24
	{	6,	5,	7,	5,	},			// block 25
	{	5,	6,	7,	6,	},			// block 26
	{	1,	0,	4,	5,	},			// block 27
	{	3,	3,	0,	0,	},			// block 28
	{	8,	7,	5,	6,	},			// block 29
	{	1,	0,	1,	6,	},			// block 30
	{	5,	5,	8,	6,	},			// block 31
	{	3,	0,	0,	3,	},			// block 32
	{	5,	6,	6,	6,	},			// block 33
	{	6,	8,	6,	6,	},			// block 34
	{	0,	1,	8,	1,	},			// block 35
	{	1,	0,	2,	6,	},			// block 36
	{	2,	0,	4,	6,	},			// block 37
	{	5,	6,	5,	6,	},			// block 38
	{	7,	8,	8,	6,	},			// block 39
	{	6,	7,	5,	5,	},			// block 40
	{	5,	6,	7,	7,	},			// block 41
	{	6,	5,	5,	8,	},			// block 42
	{	5,	6,	5,	6,	},			// block 43
	{	4,	4,	0,	0,	},			// block 44
	{	6,	4,	0,	2,	},			// block 45
	{	5,	5,	6,	6,	},			// block 46
	{	4,	0,	3,	6,	},			// block 47
	{	8,	7,	5,	6,	},			// block 48
	{	2,	5,	1,	0,	},			// block 49
	{	5,	6,	6,	6,	},			// block 50
	{	5,	5,	6,	6,	},			// block 51
	{	6,	5,	6,	6,	},			// block 52
	{	3,	0,	2,	5,	},			// block 53
	{	0,	3,	5,	1,	},			// block 54
	{	5,	5,	5,	6,	},			// block 55
	{	1,	6,	4,	0,	},			// block 56
	{	7,	4,	0,	4,	},			// block 57
	{	5,	7,	6,	5,	},			// block 58
	{	3,	6,	2,	0,	},			// block 59
	{	6,	6,	5,	5,	},			// block 60
	{	8,	4,	0,	3,	},			// block 61
	{	5,	5,	5,	7,	},			// block 62
	{	6,	6,	8,	6,	},			// block 63
	{	6,	6,	5,	5,	},			// block 64
	{	6,	6,	6,	5,	},			// block 65
	{	5,	6,	6,	8,	},			// block 66
	{	1,	0,	4,	5,	},			// block 67
	{	5,	6,	6,	5,	},			// block 68
	{	0,	3,	5,	2,	},			// block 69
	{	8,	6,	7,	7,	},			// block 70
	{	5,	4,	0,	1,	},			// block 71
	{	2,	0,	4,	6,	},			// block 72
#endif // MIDI

#ifdef MAXI
	{	0,	0,	1,	3,	},			// block 1
	{	0,	0,	1,	4,	},			// block 2
	{	0,	0,	2,	3,	},			// block 3
	{	0,	0,	3,	2,	},			// block 4
	{	0,	1,	6,	1,	},			// block 5
	{	0,	1,	7,	2,	},			// block 6
	{	0,	1,	9,	1,	},			// block 7
	{	0,	1,	9,	5,	},			// block 8
	{	0,	1,	12,	3,	},			// block 9
	{	0,	1,	14,	4,	},			// block 10
	{	0,	1,	15,	2,	},			// block 11
	{	0,	1,	19,	4,	},			// block 12
	{	0,	1,	19,	5,	},			// block 13
	{	0,	1,	21,	4,	},			// block 14
	{	0,	2,	7,	1,	},			// block 15
	{	0,	2,	8,	3,	},			// block 16
	{	0,	2,	10,	5,	},			// block 17
	{	0,	2,	13,	5,	},			// block 18
	{	0,	2,	14,	2,	},			// block 19
	{	0,	2,	15,	2,	},			// block 20
	{	0,	2,	16,	4,	},			// block 21
	{	0,	2,	17,	1,	},			// block 22
	{	0,	2,	17,	5,	},			// block 23
	{	0,	2,	18,	1,	},			// block 24
	{	0,	2,	21,	1,	},			// block 25
	{	0,	3,	6,	2,	},			// block 26
	{	0,	3,	6,	3,	},			// block 27
	{	0,	3,	7,	3,	},			// block 28
	{	0,	3,	8,	3,	},			// block 29
	{	0,	3,	14,	5,	},			// block 30
	{	0,	3,	15,	2,	},			// block 31
	{	0,	3,	18,	3,	},			// block 32
	{	0,	3,	19,	2,	},			// block 33
	{	0,	3,	19,	4,	},			// block 34
	{	0,	3,	20,	5,	},			// block 35
	{	0,	3,	22,	4,	},			// block 36
	{	0,	4,	8,	1,	},			// block 37
	{	0,	4,	11,	5,	},			// block 38
	{	0,	4,	12,	5,	},			// block 39
	{	0,	4,	13,	2,	},			// block 40
	{	0,	4,	13,	3,	},			// block 41
	{	0,	4,	15,	1,	},			// block 42
	{	0,	4,	15,	2,	},			// block 43
	{	0,	4,	15,	3,	},			// block 44
	{	0,	4,	16,	1,	},			// block 45
	{	0,	4,	18,	4,	},			// block 46
	{	0,	4,	19,	4,	},			// block 47
	{	0,	4,	20,	4,	},			// block 48
	{	0,	5,	6,	5,	},			// block 49
	{	0,	5,	7,	1,	},			// block 50
	{	0,	5,	7,	2,	},			// block 51
	{	0,	5,	9,	1,	},			// block 52
	{	0,	5,	14,	4,	},			// block 53
	{	0,	5,	16,	4,	},			// block 54
	{	0,	5,	16,	5,	},			// block 55
	{	0,	5,	19,	3,	},			// block 56
	{	0,	5,	20,	1,	},			// block 57
	{	0,	5,	20,	5,	},			// block 58
	{	0,	5,	21,	2,	},			// block 59
	{	0,	5,	22,	3,	},			// block 60
	{	6,	6,	9,	8,	},			// block 61
	{	6,	6,	10,	14,	},			// block 62
	{	6,	7,	7,	11,	},			// block 63
	{	6,	8,	6,	19,	},			// block 64
	{	6,	8,	8,	22,	},			// block 65
	{	6,	8,	10,	10,	},			// block 66
	{	6,	8, 	12,	7,	},			// block 67
	{	6,	8,	18,	9,	},			// block 68
	{	6,	8,	22,	19,	},			// block 69
	{	6,	11,	11,	14,	},			// block 70
	{	6,	11,	14,	17,	},			// block 71
	{	6,	12,	10,	8,	},			// block 72
	{	6,	12,	15,	16,	},			// block 73
	{	6,	12,	18,	15,	},			// block 74
	{	6,	12,	19,	11,	},			// block 75
	{	6,	13,	10,	15,	},			// block 76
	{	6,	13,	13,	15,	},			// block 77
	{	6,	14,	11,	20,	},			// block 78
	{	6,	14,	18,	11,	},			// block 79
	{	6,	14,	20,	21,	},			// block 80
	{	6,	15,	13,	8,	},			// block 81
	{	6,	16,	8,	8,	},			// block 82
	{	6,	16,	12,	16,	},			// block 83
	{	6,	17,	8,	13,	},			// block 84
	{	6,	17,	9,	10,	},			// block 85
	{	6,	17,	19,	17,	},			// block 86
	{	6,	17,	20,	18,	},			// block 87
	{	6,	18,	6,	21,	},			// block 88
	{	6,	18,	9,	22,	},			// block 89
	{	6,	18,	16,	20,	},			// block 90
	{	6,	19,	12,	17,	},			// block 91
	{	6,	19,	13,	15,	},			// block 92
	{	6,	19,	13,	16,	},			// block 93
	{	6,	19,	16,	21,	},			// block 94
	{	6,	19,	17,	10,	},			// block 95
	{	6,	21,	21,	11,	},			// block 96
	{	6,	22,	16,	13,	},			// block 97
	{	6,	22,	18,	19,	},			// block 98
	{	6,	22,	21,	9,	},			// block 99
	{	6,	22,	22,	21,	},			// block 100
	{	7,	7, 	17,	15,	},			// block 101
	{	7,	7, 	17,	20,	},			// block 102
	{	7,	7, 	20,	13,	},			// block 103
	{	7,	7,	22,	9,	},			// block 104
	{	7,	8, 	16,	15,	},			// block 105
	{	7,	9, 	11,	19,	},			// block 106
	{	7,	9, 	13,	19,	},			// block 107
	{	7,	9, 	16,	15,	},			// block 108
	{	7,	9,	20,	12,	},			// block 109
	{	7,	10,	15,	17,	},			// block 110
	{	7,	10,	17,	15,	},			// block 111
	{	7,	11,	18,	13,	},			// block 112
	{	7,	11,	18,	20,	},			// block 113
	{	7,	12,	10,	16,	},			// block 114
	{	7,	12,	14,	17,	},			// block 115
	{	7,	12,	17,	12,	},			// block 116
	{	7,	12,	22,	20,	},			// block 117
	{	7,	13,	11,	21,	},			// block 118
	{	7,	14,	20,	17,	},			// block 119
	{	7,	15,	19,	22,	},			// block 120
	{	7,	16,	10,	22,	},			// block 121
	{	7,	18,	9,	20,	},			// block 122
	{	7,	18,	10,	13,	},			// block 123
	{	7,	18,	18,	15,	},			// block 124
	{	7,	19,	17,	22,	},			// block 125
	{	7,	19,	21,	15,	},			// block 126
	{	7,	20,	10,	9,	},			// block 127
	{	7,	20,	13,	21,	},			// block 128
	{	7,	20,	16,	11,	},			// block 129
	{	7,	20,	18,	19,	},			// block 130
	{	7,	21,	9,	18,	},			// block 131
	{	7,	21,	17,	10,	},			// block 132
	{	7,	22,	10,	20,	},			// block 133
	{	7,	22,	12,	16,	},			// block 134
	{	7,	22,	16,	11,	},			// block 135
	{	7,	22,	20,	18,	},			// block 136
	{	8,	8,	18,	14,	},			// block 137
	{	8,	9, 	9,	11,	},			// block 138
	{	8,	9,	9,	12,	},			// block 139 HINT 0
	{	8,	9, 	9,	17,	},			// block 140
	{	8,	9, 	13,	21,	},			// block 141
	{	8,	9, 	15,	9,	},			// block 142
	{	8,	9, 	20,	17,	},			// block 143
	{	8,	9, 	21,	21,	},			// block 144
	{	8,	10,	11,	16,	},			// block 145
	{	8,	11,	8,	17,	},			// block 146
	{	8,	11,	8,	22,	},			// block 147
	{	8,	11,	11,	10,	},			// block 148
	{	8,	11,	15,	17,	},			// block 149
	{	8,	13,	9,	12,	},			// block 150
	{	8,	13,	16,	22,	},			// block 151
	{	8,	14,	12,	12,	},			// block 152
	{	8,	14,	12,	13,	},			// block 153
	{	8,	14,	22,	20,	},			// block 154
	{	8,	16,	14,	14,	},			// block 155
	{	8,	16,	14,	17,	},			// block 156
	{	8,	16,	22,	14,	},			// block 157
	{	8,	18,	14,	20,	},			// block 158
	{	8,	18,	19,	9,	},			// block 159
	{	8,	19,	21,	21,	},			// block 160
	{	8,	20,	9,	18,	},			// block 161
	{	8,	20,	10,	18,	},			// block 162
	{	8,	22,	15,	12,	},			// block 163
	{	8,	22,	16,	20,	},			// block 164
	{	9,	10,	11,	16,	},			// block 165
	{	9,	10,	16,	19,	},			// block 166
	{	9,	12,	11,	11,	},			// block 167
	{	9,	13,	12,	15,	},			// block 168
	{	9,	13,	13,	21,	},			// block 169
	{	9,	14,	16,	19,	},			// block 170
	{	9,	14,	18,	20,	},			// block 171
	{	9,	14,	21,	12,	},			// block 172
	{	9,	15,	15,	15,	},			// block 173
	{	9,	15,	17,	18,	},			// block 174
	{	9,	16,	14,	21,	},			// block 175
	{	9,	17,	14,	13,	},			// block 176
	{	9,	18,	10,	16,	},			// block 177
	{	9,	19,	17,	20,	},			// block 178
	{	9,	19,	19,	15,	},			// block 179
	{	9,	20,	14,	20,	},			// block 180
	{	9,	21,	12,	20,	},			// block 181 HINT 1
	{	9,	21,	14,	12,	},			// block 182
	{	10,	10,	13,	19,	},			// block 183
	{	10,	11,	22,	14,	},			// block 184
	{	10,	12,	13,	17,	},			// block 185
	{	10,	12,	19,	19,	},			// block 186
	{	10,	13,	21,	14,	},			// block 187
	{	10,	14,	10,	21,	},			// block 188
	{	10,	14,	11,	13,	},			// block 189
	{	10,	14,	20,	13,	},			// block 190
	{	10,	15,	11,	11,	},			// block 191
	{	10,	15,	15,	18,	},			// block 192
	{	10,	16,	12,	14,	},			// block 193
	{	10,	17,	21,	12,	},			// block 194
	{	10,	17,	21,	22,	},			// block 195
	{	10,	17,	22,	15,	},			// block 196
	{	10,	18,	12,	22,	},			// block 197
	{	10,	18,	13,	19,	},			// block 198
	{	10,	18,	18,	18,	},			// block 199
	{	10,	19,	13,	11,	},			// block 190
	{	10,	20,	21,	19,	},			// block 201
	{	10,	20,	22,	14,	},			// block 202
	{	10,	21,	17,	13,	},			// block 203
	{	10,	21,	17,	19,	},			// block 204
	{	10,	21,	20,	11,	},			// block 205
	{	10,	21,	22,	21,	},			// block 206
	{	11,	11,	22,	14,	},			// block 207
	{	11,	12,	13,	22,	},			// block 208
	{	11,	12,	19,	15,	},			// block 209
	{	11,	14,	12,	13,	},			// block 210
	{	11,	14,	20,	15,	},			// block 211
	{	11,	15,	11,	20,	},			// block 212
	{	11,	16,	19,	19,	},			// block 213
	{	11,	17,	11,	18,	},			// block 214
	{	11,	17,	16,	22,	},			// block 215
	{	11,	17,	22,	21,	},			// block 216
	{	11,	18,	13,	15,	},			// block 217
	{	11,	20,	16,	17,	},			// block 218
	{	11,	21,	12,	16,	},			// block 219
	{	11,	22,	12,	20,	},			// block 220
	{	11,	22,	21,	21,	},			// block 221
	{	11,	22,	21,	22,	},			// block 222
	{	12,	12,	17,	22,	},			// block 223
	{	12,	12,	18,	14,	},			// block 224
	{	12,	12,	20,	15,	},			// block 225
	{	12,	14,	13,	15,	},			// block 226
	{	12,	14,	17,	17,	},			// block 227
	{	12,	16,	13,	19,	},			// block 228
	{	12,	18,	14,	22,	},			// block 229
	{	12,	18,	20,	19,	},			// block 230
	{	12,	19,	17,	18,	},			// block 231
	{	12,	19,	17,	21,	},			// block 232
	{	13,	13,	13,	18,	},			// block 233
	{	13,	14,	20,	16,	},			// block 234
	{	13,	16,	14,	16,	},			// block 235
	{	13,	16,	14,	18,	},			// block 236
	{	13,	16,	17,	15,	},			// block 237
	{	13,	17,	16,	20,	},			// block 238
	{	13,	18,	15,	22,	},			// block 239
	{	13,	18,	21,	15,	},			// block 240
	{	13,	19,	14,	21,	},			// block 241
	{	13,	19,	16,	21,	},			// block 242
	{	14,	15,	15,	17,	},			// block 243
	{	14,	15,	18,	19,	},			// block 244
	{	14,	16,	22,	18,	},			// block 245
	{	14,	21,	20,	22,	},			// block 246
	{	15,	15,	21,	22,	},			// block 247
	{	15,	16,	17,	16,	},			// block 248
	{	15,	17,	16,	21,	},			// block 249
	{	15,	18,	20,	21,	},			// block 250
	{	16,	16,	22,	19,	},			// block 251
	{	16,	17,	19,	17,	},			// block 252
	{	17,	19,	20,	18,	},			// block 253
	{	18,	20,	21,	20,	},			// block 254
	{	18,	22,	20,	22,	},			// block 255 HINT 2
	{	19,	22,	21,	22,	},			// block 256
#endif // MAXI
};
